def main() -> None:
    """Main function."""
    auth = {
        'server': 'http://192.168.31.78',
        'username': 'root@ro.ot',
        'password': '65Beforeth!',
        'proxy': 'http://127.0.0.1:8080',
    }

    hive: Hive = Hive().connect(**auth)
    project_id = '5c188d0f-f854-4030-8b98-01ace06b1fc8'
    issue_id = '506f96b1-ef33-4881-9a18-799cd8c5b3f0'
    # print(hive.get_projects())
    # print(hive.get_issues('5c188d0f-f854-4030-8b98-01ace06b1fc8'))
    # print(hive.get_file('5c188d0f-f854-4030-8b98-01ace06b1fc8', '4f03056c-cfa6-4092-94cb-625b5a17e82c'))
    hive.update_issue(project_id, issue_id, status='retest_required')
    # print(hive.get_project('5c188d0f-f854-4030-8b98-01ace06b1fc8'))
    # pprint(hive.get_projects(), indent=4)
    # project_dict: Dict = json.load(open('project.json', 'r', encoding='utf-8'))
    # project: Project = Project(**project_dict)
    # print(project)


if __name__ == "__main__":
    main()



from enum import StrEnum, auto
from typing import Type, Dict, Optional, Self

from Hive.http_client.http_client import HTTPClient
from Hive.rest.exceptions import RestConnectionError


class State(StrEnum):
    """Enumeration of states."""
    NOT_CONNECTED = auto()
    CONNECTED = auto()
    DISCONNECTED = auto()


Route = StrEnum('Route', {
    'SESSION': 'session',
    'AUTH': 'session',
    'PROJECT': 'project',
    'FILES': 'files',

})


class BaseHiveClient:
    base_url: str = None
    route: Type[Route] = None

    def __init__(self):
        """Initialize RestHive."""
        self.state: State = State.NOT_CONNECTED
        self.http_client: HTTPClient = HTTPClient()

    @staticmethod
    def _get_routes() -> Type[Route]:
        """Get routes from routes.py."""
        return Route

    def _update_base_url(self, server: str, port: int, verify: bool) -> True:
        """Prepare base url."""
        if not server.startswith('http'):
            scheme = 'https://' if verify else 'http://'
            if not port:
                port = 443 if verify else 80
            self.base_url = f"{scheme}{server}:{port}/api".strip('/ ')
        else:
            self.base_url = server.strip('/ ')
            self.base_url += '/api' if not self.base_url.endswith('/api') else ''
        return True

    def _update_http_client(self, verify: bool, proxies: Optional[Dict] = None, proxy: Optional[str] = None) -> True:
        """Prepare http client."""
        proxies = proxies or {'http': proxy, 'https': proxy} or {}
        self.http_client.update_params(verify=verify, proxies=proxies)
        self.http_client.add_headers({'Content-Type': 'application/json'})
        return True

    def _update_routes(self, server: str, port: int, verify: bool) -> True:
        """Prepare urls."""
        self._update_base_url(server, port, verify)
        self.route = StrEnum('Url', {key: f'{self.base_url}/{value.strip("/")}'
                                     for key, value in Route.__members__.items()})
        return True

    def _preconnect_update(self, **kwargs) -> True:
        """Update settings before connecting."""
        self._update_routes(**{'server': kwargs['server'],
                               'port': kwargs['port'],
                               'verify': kwargs['verify'],
                               })

        self._update_http_client(**{'verify': (kwargs['verify']),
                                    'proxies': kwargs.get('proxies'),
                                    'proxy': kwargs.get('proxy')
                                    })

        return True

    def _authenticate(self, username: str, password: str) -> None:
        """Authenticate in Hive."""
        response = self.http_client.session.post(self.route.AUTH, json={
            'userLogin': username,
            'userPassword': password,
        })

        try:
            cookie = response.cookies.get('BSESSIONID')
            self.http_client.add_headers({'Cookie': f'BSESSIONID={cookie}'})
            self.state = State.CONNECTED
        except AttributeError:
            raise RestConnectionError('Could not get authentication cookie. Something wrong with credentials '
                                      'or server.')

    def connect(self,
                server: str,
                username: str,
                password: str,
                port: Optional[int] = None,
                verify: bool = True,
                proxies: Optional[Dict] = None,
                proxy: Optional[str] = None,
                ) -> Self:
        """Method connects to Hive instance."""
        self._preconnect_update(server=server, port=port, verify=verify, proxies=proxies, proxy=proxy)
        self._authenticate(username, password)
        return self



__description__ = "Small wrapper for HexWay Hive API "
__author__ = "@Cur1iosity"
__email__ = "Cur1iosity@protonmail.com"
__licence__ = "MIT"
__version__ = "0.1.0"

from hive_client import Hive

__all__ = [
        'Hive',
]



from datetime import datetime
from typing import Dict, Optional, Any
from uuid import UUID

import pydantic


class Project(pydantic.BaseModel):
    id: str = pydantic.Field(alias='projectId')
    name: str = pydantic.Field(alias='projectName')
    description: Optional[str] = pydantic.Field(default=None, alias='projectDescription')
    additional_fields: Optional[Dict] = pydantic.Field(default=None, alias='data')

    slug: Optional[str] = pydantic.Field(default=None, alias='projectSlug')

    group_id: str = pydantic.Field(alias='projectGroupId')
    full_slug: str = pydantic.Field(default=None, alias='projectFullSlug')

    report_template: Optional[str] = pydantic.Field(default=None, alias='defaultReportTemplateName')
    scope: Optional[str] = pydantic.Field(default=None, alias='projectScope')
    permissions: Optional[Dict] = pydantic.Field(default=None, alias='projectPermissions')

    is_archived: bool = pydantic.Field(default=None, alias='projectIsArchived')
    archive_date: Optional[str] = pydantic.Field(default=None, alias='projectArchiveDate')

    create_date: Optional[str] = pydantic.Field(default=None, alias='projectCreateDate')
    update_date: Optional[str] = pydantic.Field(default=None, alias='projectUpdateDate')
    start_date: Optional[str] = pydantic.Field(default=None, alias='projectStartDate')
    end_date: Optional[str] = pydantic.Field(default=None, alias='projectEndDate')

    application_connect: Optional[str] = pydantic.Field(default=None, alias='applicationConnect', repr=False)
    connection_id: Optional[str] = pydantic.Field(default=None, alias='projectConnectionId', repr=False)
    connection_name: Optional[str] = pydantic.Field(default=None, alias='connectionName', repr=False)
    last_incoming_pong: Optional[str] = pydantic.Field(default=None, alias='lastIncomingPong', repr=False)
    last_outgoing_ping: Optional[str] = pydantic.Field(default=None, alias='lastOutgoingPing', repr=False)
    connection: Optional[Any] = pydantic.Field(default=None, alias='hive', repr=False, exclude=True)

    @pydantic.field_validator('id', 'group_id', mode='before')
    def validate_id(cls, value: str) -> str:
        """Validate hive id-type field."""
        try:
            UUID(value)
        except ValueError:
            raise ValueError('Invalid project id.')
        return value

    @pydantic.field_validator('name', 'description', mode='before')
    def validate_text_field(cls, value: str) -> str:
        """Validate text field."""
        value = value.strip('\n')
        return value if value.endswith('__') else value.strip('__')

    @pydantic.field_validator('create_date', 'update_date', 'start_date', 'end_date', mode='before')
    def validate_datetime(cls, value: str) -> str:
        """Validate datetime field."""
        try:
            dt: datetime = datetime.strptime(value, '%Y-%m-%dT%H:%M:%S.%fZ')
        except ValueError:
            raise ValueError('Invalid datetime.')
        return datetime.strftime(dt, '%d-%m-%Y %H:%M:%S')

    @pydantic.model_serializer
    def serialize(self) -> Dict:
        """Serialize object to dict."""
        additional_fields = (dump := self.model_dump()).pop('additional_fields') or {}
        return dump | additional_fields

    pydantic.ConfigDict(
        populate_by_name=True,
    )

    def __str__(self) -> str:
        """Return string representation of object."""
        return f'{self.__class__.__name__}({"".join(f"{key}={value}, " for key, value in self.model_dump().items())})'



import datetime
from typing import Optional, List, Dict, Any
from uuid import UUID

import pydantic


class Issue(pydantic.BaseModel):
    uuid: str = pydantic.Field(alias='uuid')
    name: str = pydantic.Field(alias='name')
    hostnames: List[str] = pydantic.Field(alias='hostnames')
    ip_addresses: List[str] = pydantic.Field(alias='ips')
    cvss_vector: Optional[str] = pydantic.Field( default=None, alias='cvssVector')
    cvss_score: Optional[float] = pydantic.Field(default=None, alias='cvssScore')
    weakness_type: Optional[str] = pydantic.Field(default=None, alias='weaknessType')
    checkmark_count: int = pydantic.Field(alias='checkmarkCount')
    issueSource: Optional[str] = pydantic.Field(default=None, alias='issueSourceType')

    probability_score: Optional[float] = pydantic.Field(default=None, alias='probabilityScore')
    criticality_score: Optional[float] = pydantic.Field(default=None, alias='criticalityScore')
    total_score: Optional[float] = pydantic.Field(default=None, alias='totalScore')

    labels: Optional[List[str]] = pydantic.Field(default=None, alias='labels')
    post_time: Optional[str] = pydantic.Field(default=None, alias='postTime')

    additional_fields: Optional[Dict] = pydantic.Field(default=None, alias='additionalFields')

    note_count: Optional[int] = pydantic.Field(default=None, alias='noteCount')
    notes: Optional[List] = pydantic.Field(default=None, alias='notes')

    files: Optional[List[Dict]] = pydantic.Field(default=None, alias='files')
    requests: Optional[List[Dict]] = pydantic.Field(default=None, alias='requests')

    status: str = pydantic.Field(alias='status')

    internal_id: int = pydantic.Field(alias='id')
    sync_status: Optional[str] = pydantic.Field(default=None, alias='syncStatus')
    node_id: Optional[str] = pydantic.Field(default=None, alias='nodeId')
    parent_id: Optional[str] = pydantic.Field(default=None, alias='parentId')
    message_count_brig: Optional[int] = pydantic.Field(default=0, alias='messageCountBrig')
    message_count_frigate: Optional[int] = pydantic.Field(default=0, alias='messageCountFrigate')

    creator_uuid: str = pydantic.Field(alias='creatorUuid')
    connection: Optional[Any] = pydantic.Field(default=None, alias='hive', repr=False, exclude=True)

    @pydantic.field_validator('name', mode='before')
    def validate_text_field(cls, value: str) -> str:
        """Validate text field."""
        value = value.strip('\n')
        return value if value.endswith('__') else value.strip('__')

    @pydantic.field_validator('post_time', mode='before')
    def validate_datetime(cls, value: str) -> str:
        """Validate datetime field."""
        try:
            dt: datetime.datetime = datetime.datetime.strptime(value, '%Y-%m-%dT%H:%M:%S.%fZ')
        except ValueError:
            raise ValueError('Invalid datetime.')
        return datetime.datetime.strftime(dt, '%d-%m-%Y %H:%M:%S')

    @pydantic.field_validator('uuid', 'creator_uuid', mode='before')
    def validate_id(cls, value: str) -> str:
        """Validate hive id-type field."""
        try:
            UUID(value)
        except ValueError:
            raise ValueError('Invalid vulnerability id.')
        return value

    @pydantic.model_serializer
    def serialize(self) -> Dict:
        """Serialize object to dict."""
        additional_fields = (dump := self.model_dump()).pop('additional_fields') or {}
        return dump | additional_fields

    pydantic.ConfigDict(
        populate_by_name=True,
    )

    def __str__(self) -> str:
        """Return string representation of object."""
        return f'{self.__class__.__name__}({"".join(f"{key}={value}, " for key, value in self.model_dump().items())})'



from PyCMDB.models.CIS import *
from typing import Type, Any, List
from Hive import models

objects = [obj for name, obj in vars(models).items() if isinstance(obj, type)]
object_mapper = {x.__name__: x for x in objects}


class Factory:
    """Factory that creates items."""
    @staticmethod
    def create_instance(class_type: Type, **kwargs) -> Any:
        """Create instance of class."""
        return class_type(**kwargs)

    def get_model_field_names(self, class_name: str) -> List[str]:
        """Get model field names."""
        return [x for x in set(self.get_attributes(class_name) + self.get_aliases(class_name)) if x]

    @staticmethod
    def get_attributes(class_name: str) -> List[str]:
        """Get attributes of class."""
        try:
            return list(object_mapper.get(camel_case(class_name)).model_fields.keys())
        except AttributeError:
            raise ModelNotFoundError(class_name)

    @staticmethod
    def get_aliases(class_name: str) -> List[str]:
        """Get aliases of class."""
        try:
            return [value.alias for value in object_mapper.get(camel_case(class_name)).model_fields.values()]
        except AttributeError:
            raise ModelNotFoundError(class_name)

    def __call__(self, class_name: str, **kwargs) -> Any:
        """Call creating instance of class."""
        try:
            class_type = object_mapper.get(camel_case(class_name))
            return self.create_instance(class_type, **kwargs)
        except TypeError:
            raise ModelNotFoundError(class_name)


class FactoryError(Exception):
    """Factory error."""
    def __init__(self, message: str = ''):
        self.message = message


class ModelNotFoundError(FactoryError):
    """Model not found error."""
    def __init__(self, message: str = ''):
        super().__init__(message=f"Model '{message}' not found. Check __init__.py in models package")


def camel_case(snake_str: str) -> str:
    """Convert snake string to camel string."""
    components = snake_str.split('_')
    return ''.join(x.title() for x in components)



from typing import TypeVar

from Hive.enums import ObjectType
from Hive.meta import wrap
from Hive.rest.base_client import BaseHiveClient

T = TypeVar('T', bound='HiveObject')


class Hive(BaseHiveClient):
    """Hive client."""
    @wrap(ObjectType.PROJECT, many=True)
    def get_projects(self) -> T:
        """Get projects."""
        return self.http_client.get(self.route.PROJECT)

    @wrap(ObjectType.PROJECT)
    def get_project(self, project_id: str) -> T:
        """Get project."""
        return self.http_client.get(f'{self.route.PROJECT}/{project_id}')

    @wrap(ObjectType.ISSUE, many=True)
    def get_issues(self, project_id: str, offset: int = 0, limit: int = -1) -> T:
        """Get project vulnerabilities."""
        kwargs = {'json': {}, 'params': {"offset": offset, "limit": limit}}
        return self.http_client.post(f'{self.route.PROJECT}/{project_id}/graph/issue_list', **kwargs).get('items')

    def update_issue(self, project_id: str, issue_id: str, **kwargs) -> T:
        """Update issue."""
        return self.http_client.patch(f'{self.route.PROJECT}/{project_id}/graph/issues/{issue_id}', json=kwargs)

    def get_file(self, project_id: str,  file_id: str) -> bytes:
        """Get file."""
        return self.http_client.get(f"{self.route.PROJECT}/{project_id}/graph/file/{file_id}")



from typing import List, Union, Callable
from Hive.enums import ObjectType
from Hive.models.factory import Factory
import functools

factory = Factory()


def wrap(to_object: ObjectType, many: bool = False) -> Callable:
    """Wrap function to convert response to object."""
    def decorator(func) -> Callable:
        @functools.wraps(func)
        def wrapper(instance, *args, **kwargs) -> Union[List[ObjectType], ObjectType]:
            if many:
                return [factory(to_object, **x) for x in func(instance, *args, **kwargs)]
            else:
                return factory(to_object, **func(instance, *args, **kwargs))
        return wrapper
    return decorator


def json(func):
    """Wrap function to convert response to json."""
    @functools.wraps(func)
    def wrapper(*args, **kwargs) -> Union[List[dict], dict]:
        return func(*args, **kwargs).json()
    return wrapper


def content(func):
    """Wrap function to convert response to content."""
    @functools.wraps(func)
    def wrapper(*args, **kwargs) -> Union[List[dict], dict]:
        return func(*args, **kwargs).content
    return wrapper
